
#include "CyUsbCom.h"
#include <rdp_base\LogWraper.h>
#include <rdp_base\MsgHolder.h>

CyUsbCom::CyUsbCom( )
{
	cyDevice = new CCyUSBDevice();
	deviceCount = 0;
}

CyUsbCom::~CyUsbCom(void)
{	
	delete cyDevice;
	deviceCount = 0;
}

BOOL CyUsbCom::OpenUsbCom(std::string portStr)
{
	if (cyDevice->IsOpen() && cyDevice->DeviceName == portStr)
	{
		return true;
	}
	deviceCount = cyDevice->DeviceCount();
	for (int deviceIndex = 0; deviceIndex < deviceCount; deviceIndex++)
	{
		if (cyDevice->Open(deviceIndex) && cyDevice->DeviceName == portStr)
		{
			return true;
		}
	}
	return false;
}

void CyUsbCom::CloseUsbCom()
{
	if (cyDevice->BulkInEndPt)
	{
		cyDevice->BulkInEndPt->Abort();
	}
	
	if (cyDevice->BulkOutEndPt)
	{
		cyDevice->BulkOutEndPt->Abort();
	}
	cyDevice->Close();
}

BOOL CyUsbCom::SentData( BYTE *pSource, unsigned Length )
{

	if (!cyDevice->BulkOutEndPt)
	{
		return FALSE;
	}
	int errorCode = -1;
	OVERLAPPED outOvLap, inOvLap;
	outOvLap.hEvent  = CreateEvent(NULL, false, false, _T("CYUSB_OUT"));

	
	UCHAR  *outContext = cyDevice->BulkOutEndPt->BeginDataXfer(pSource, Length, &outOvLap);
	if (!cyDevice->BulkOutEndPt->WaitForXfer(&outOvLap,10000))
	{
		errorCode = GetLastError();
		return false;
	}
	
	if (!cyDevice->BulkOutEndPt->FinishDataXfer(pSource, (LONG &)Length, &outOvLap,outContext))
	{
		errorCode = GetLastError();
		return false;
	}
	
	CloseHandle(outOvLap.hEvent);
	return true;
}

BOOL CyUsbCom::ResetFpgaFifo()
{
	USHORT reValue = 0;
	return VendorRequest(ResetFpgaFio, 0x00, reValue);
}

BOOL CyUsbCom::GetDdrWriteAdd( UINT &ddrAdd )
{
	USHORT reValueL;
	USHORT reValueH;

	if (!GetParameter( mig_wr_addr_rdbackL, reValueL))
	{
		return FALSE;
	}

	if (!GetParameter( mig_wr_addr_rdbackH, reValueH))
	{
		return FALSE;
	}

	ddrAdd = (UINT)(reValueH) << 16;
	ddrAdd = ddrAdd + reValueL;
	return TRUE;
}

BOOL CyUsbCom::GetDdrReadAdd( UINT &ddrAdd )
{
	USHORT reValueL;
	USHORT reValueH;

	if (!GetParameter( mig_rd_addr_rdbackL, reValueL))
	{
		return FALSE;
	}

	if (!GetParameter( mig_rd_addr_rdbackH, reValueH))
	{
		return FALSE;
	}

	ddrAdd = (UINT)(reValueH) << 16;
	ddrAdd = ddrAdd + reValueL;
	return TRUE;
}

BOOL CyUsbCom::SetParameter( USHORT bReg, USHORT bValue )
{
	boost::mutex::scoped_lock lk(mt);
	USHORT reValue = 0;
	if (!VendorRequest(SetReg, bReg, reValue))
	{
		return FALSE;
	}
	return VendorRequest(SetRegData, bValue, reValue);
}

BOOL CyUsbCom::GetParameter( USHORT bReg, USHORT &retValue )
{
	boost::mutex::scoped_lock lk(mt);
	USHORT reValue = 0;
	if (!VendorRequest(SetReg, bReg, reValue))
	{
		return FALSE;
	}
	return VendorRequest(GetRegData, 0x0, retValue);
}

BOOL CyUsbCom::VendorRequest( UCHAR bRequest, USHORT bValue, USHORT &retValue )
{	
	CCyControlEndPoint *ept = cyDevice->ControlEndPt;
	if (ept == NULL)
	{
		RDP_LOG_ERROR("VendorRequest error! Please Check DMD!");
		SEND_MESSAGE(L"дDMDʧܣDMD");
		return FALSE;
	}

	OVERLAPPED OvLap;
	OvLap.hEvent  = CreateEvent(NULL, false, false, _T("CYUSB_CTL"));
	BYTE buffer[128];
	LONG  length   = 128;

	ept->ReqCode   = (UCHAR)bRequest;
	ept->Value     = bValue;
	ept->Index     = 0;
	ept->Direction = DIR_FROM_DEVICE;
	ept->ReqType   = REQ_VENDOR;
	ept->Target    = TGT_DEVICE;
	
	PUCHAR Context = ept->BeginDataXfer(buffer, length, &OvLap);
	ept->WaitForXfer(&OvLap,100);
	ept->FinishDataXfer(buffer, length, &OvLap,Context);
	CloseHandle(OvLap.hEvent);

	if (buffer[0] == bRequest)
	{
		BYTE *tmp = (BYTE*)(&retValue);
		for(int i = 1, j = 0; i < 3; i++, j++)
		{
			tmp[j] = buffer[i];
		}
		return TRUE;		
	}
	return FALSE;
}

BOOL CyUsbCom::VendorRequest( UCHAR bRequest, BYTE* buffer,LONG length)
{	
	CCyControlEndPoint  *ept = cyDevice->ControlEndPt;
	OVERLAPPED OvLap;
	OvLap.hEvent  = CreateEvent(NULL, false, false, _T("CYUSB_CTL"));
	ept->ReqCode   = (UCHAR)bRequest;
	ept->Value     = 0;
	ept->Index     = 0;
	ept->Direction = DIR_TO_DEVICE;
	ept->ReqType   = REQ_VENDOR;
	ept->Target    = TGT_DEVICE;

	PUCHAR Context = ept->BeginDataXfer(buffer, length, &OvLap);
	ept->WaitForXfer(&OvLap,100);
	ept->FinishDataXfer(buffer, length, &OvLap,Context);
	CloseHandle(OvLap.hEvent);
	
	USHORT tempVal = 0;
	if (buffer[0] == bRequest)
	{
		BYTE *tmp = (BYTE*)(&tempVal);
		for(int i = 1, j = 0; i < 3; i++, j++)
		{
			tmp[j] = buffer[i];
		}
		return TRUE;		
	}
	return FALSE;
}


